#ifndef PROPERTY_H
#define PROPERTY_H

#include <assert.h>
#include <iostream>

#define READ_ONLY 1
#define WRITE_ONLY 2
#define READ_WRITE 3


//! Simula le propriet in C++
/*! Una propriet permette di accedere ad un attributo privato tramite due funzioni, una di Set per la scrittura e una di Get per la lettura. Queste due funzioni vengono invocate automaticamente utilizzando l'attibuto privato e l'operatore di assegnamento (=).
    
    Copyrights Emad Barsoum (ebarsoum@msn.com) 2003. All rights reserved.
    _____________________________________________________________________

     VERSION: 1.0

     LICENSE: You are free to change or modify or redistribute the code, just keep the header. And you can use this class in any application you want without any warranty.
*/
template<typename Container, typename ValueType, int nPropType>
class Property
{
public:

    //! Costruttore
    Property()
    {
        m_cObject = NULL;
        set = NULL;
        get = NULL;
    }
    
    //! Inizializza il puntatore alla classe che contiene la propriet
    void SetContainer(Container* cObject)
    {
        m_cObject = cObject;
    }
    
    //! Inizializza la funzione di set, utilizzata per modificare il valore
    void Setter(void (Container::*pSet)(ValueType value))
    {
        if ((nPropType == WRITE_ONLY) || (nPropType == READ_WRITE))
            set = pSet;
        else
            set = NULL;
    }
    
    //! Inizializza la funzione di set, utilizzata per leggere il valore
    void Getter(ValueType (Container::*pGet)())
    {
        if ((nPropType == READ_ONLY) || (nPropType == READ_WRITE))
            get = pGet;
        else
            get = NULL;
    }
    
    //! Overload dell'operatore di assegnamento "="
    ValueType operator =(const ValueType& value)
    {
        assert(m_cObject != NULL);
        assert(set != NULL);
        (m_cObject->*set)(value);
        return value;
    }
    
    //! Operatore usato per le conversioni con i tipi interni
    operator ValueType()
    {
        assert(m_cObject != NULL);
        assert(get != NULL);
        return (m_cObject->*get)();
    }

private:

    //! Puntatore all'oggetto che contiene la propriet
    Container* m_cObject;
    
    //! Puntatore alla funzione Set
    void (Container::*set)(ValueType value);
    
    //! Puntatore alla funzione Get
    ValueType (Container::*get)();
};

#endif
